/*
 * ZeroTier One - Network Virtualization Everywhere
 * Copyright (C) 2011-2015  ZeroTier, Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * --
 *
 * ZeroTier may be used and distributed under the terms of the GPLv3, which
 * are available at: http://www.gnu.org/licenses/gpl-3.0.html
 *
 * If you would like to embed ZeroTier into a commercial application or
 * redistribute it in a modified binary form, please contact ZeroTier Networks
 * LLC. Start here: http://www.zerotier.com/
 */

package com.zerotier.sdk;

import java.net.InetSocketAddress;
import java.lang.String;

/**
 * Interface to handle callbacks for ZeroTier One events.
 */
public interface EventListener {
    /**
     * Callback for events with no other associated metadata
     * 
     * @param event {@link Event} enum
     */
    public void onEvent(Event event);
    
    /**
     * Callback for network error events: {@link Event.EVENT_AUTHENTICATION_FAILURE}, {link Event.EVENT_INVALID_PACKET}
     *
     * @param event {@link Event} enum
     * @param source {@link InetSocketAddress} containing the origin address of the packet
     */
    public void onNetworkError(Event event, InetSocketAddress source);

    /**
     * Callback when the node detects that it's out of date.
     *
     * @param newVersion {@link Version} object with the latest version of ZeroTier One
     */
    public void onOutOfDate(Version newVersion);

    /**
     * Trace messages
     * 
     * <p>These events are only generated if the underlying ZeroTierOne SDK is a TRACE-enabled build.</p>
     *
     * @param message the trace message
     */
    public void onTrace(String message);
}
